@@ -37,6 +37,8 @@
 #include <unistd.h>
 #include <signal.h>
 
+#define LINE_LENGTH 4096 /* max line length we'll grok */
+
 typedef struct {
 	GnomeVFSMethodHandle method_handle;
 	GnomeVFSURI *uri;
@@ -153,17 +155,21 @@ ssh_connect (SshHandle **handle_return,
 	char ** argv;
 	SshHandle *handle;
 	char *command_line;
+	const gchar *username;
 	int argc;
 	GError *gerror = NULL;
 
+	username = gnome_vfs_uri_get_user_name(uri);
+	if (username == NULL) {
+		username = g_get_user_name();
+	}
 	
-	command_line  = g_strconcat ("ssh -oBatchmode=yes -x -l ", 
-				     gnome_vfs_uri_get_user_name (uri),
+	command_line  = g_strconcat ("ssh -oBatchMode=yes -x -l ", 
+				     username,
 				     " ", gnome_vfs_uri_get_host_name (uri),
-				     " ", command,
+				     " ", "\"LC_ALL=C;", command,"\"",
 				     NULL);
 
-
 	g_shell_parse_argv (command_line, &argc, &argv, &gerror);
 	g_free (command_line);
 	if (gerror) {
@@ -256,26 +262,6 @@ ssh_write (SshHandle *handle,
 }
 
 #if 0
-static char *ssh_escape (const char *string)
-{
-	char *new_str;
-	int i,j;
-	
-	new_str = g_malloc0 (strlen (string)*2+3);
-
-	new_str[0]='\'';
-
-	for (i=0,j=1; string[i] != '\0'; i++, j++) {
-		if (string[i] == '\'') {
-			new_str[j] = '\\';
-			j++;
-		}
-		new_str[j] = string[i];
-	}
-
-	return new_str;
-}
-
 static GnomeVFSResult
 ssh_send (SshHandle *handle,
 	  const char *string)
@@ -305,27 +291,23 @@ do_open (GnomeVFSMethod *method,
 	GnomeVFSResult result = GNOME_VFS_OK;
 	char *cmd;
 	SshHandle *handle = NULL;
-	char *name;
-
-	/* FIXME: escape for shell */
-	name = gnome_vfs_unescape_string (uri->text, G_DIR_SEPARATOR_S);
 
 	if (mode == GNOME_VFS_OPEN_READ) {
-		/* FIXME: escape for shell */
-		cmd = g_strdup_printf ("cat '%s'", name);
+		char *name;
+		char *quoted_name;
+		name = gnome_vfs_unescape_string (uri->text, 
+						  G_DIR_SEPARATOR_S);
+		quoted_name = g_shell_quote (name);
+		g_free (name);
+		
+		cmd = g_strdup_printf ("cat %s", quoted_name);
 		result = ssh_connect (&handle, uri, cmd);
 		g_free (cmd);
-
+		g_free (quoted_name);
 		if (result != GNOME_VFS_OK) {
-			g_free (name);
 			return result;
 		}
-
-	} else if (mode == GNOME_VFS_OPEN_WRITE) {
-		g_free (name);
-		return GNOME_VFS_ERROR_INVALID_OPEN_MODE;
 	} else {
-		g_free (name);
 		return GNOME_VFS_ERROR_INVALID_OPEN_MODE;
 	}
 	
@@ -333,9 +315,7 @@ do_open (GnomeVFSMethod *method,
 	handle->type = SSH_FILE;
 	*method_handle = (GnomeVFSMethodHandle *)handle;
 
-	g_free (name);
-
-	return result;
+	return GNOME_VFS_OK;
 }
 
 static GnomeVFSResult   
@@ -351,23 +331,22 @@ do_create (GnomeVFSMethod *method,
 	char *cmd;
 	GnomeVFSResult result;
 	char *name;
-
-	/* FIXME: escape for shell */
-	name = gnome_vfs_unescape_string (uri->text, G_DIR_SEPARATOR_S);
+	char *quoted_name;
 
 	if (mode != GNOME_VFS_OPEN_WRITE) {
-		g_free (name);
 		return GNOME_VFS_ERROR_INVALID_OPEN_MODE;
 	}
 
+	name = gnome_vfs_unescape_string (uri->text, G_DIR_SEPARATOR_S);
+	quoted_name = g_shell_quote (name);
 
-	/* FIXME: escape for shell */
-	cmd = g_strdup_printf ("cat > '%s'", name);
+	cmd = g_strdup_printf ("cat > %s", quoted_name);
 	result = ssh_connect (&handle, uri, cmd);
 	g_free (cmd);
+	g_free (name);
+	g_free (quoted_name);
 
 	if (result != GNOME_VFS_OK) {
-		g_free (name);
 		return result;
 	}
 
@@ -377,7 +356,6 @@ do_create (GnomeVFSMethod *method,
 	handle->type = SSH_FILE;
 	*method_handle = (GnomeVFSMethodHandle *)handle;
 
-	g_free (name);
 	return result;
 }
 
@@ -397,8 +375,14 @@ do_read (GnomeVFSMethod *method,
 	 GnomeVFSFileSize *bytes_read,
 	 GnomeVFSContext *context)
 {
-	return ssh_read ((SshHandle *)method_handle, buffer, num_bytes,
+	GnomeVFSResult result;
+
+	result =  ssh_read ((SshHandle *)method_handle, buffer, num_bytes,
 			bytes_read);
+	if (*bytes_read == 0) {	
+		result = GNOME_VFS_ERROR_EOF;
+	}
+	return result;
 }
 
 /* alternative impl:
@@ -427,17 +411,19 @@ do_open_directory (GnomeVFSMethod *method,
 	char *cmd = NULL;
 	GnomeVFSResult result;
 	char *name;
+	char *quoted_name;
 
-	/* FIXME: escape for shell */
 	name = gnome_vfs_unescape_string (uri->text, G_DIR_SEPARATOR_S);
+	quoted_name = g_shell_quote (name);
 
-	if (strlen (name) > 0) {
-		cmd = g_strdup_printf ("ls -l '%s'", name);
+	if (*name != '\0') {
+		cmd = g_strdup_printf ("ls -l %s", quoted_name);
 	} else {
 		cmd = g_strdup_printf ("ls -l '/'");
 	}
 
 	result = ssh_connect (&handle, uri, cmd);
+	g_free (quoted_name);
 	g_free (name);
 	g_free (cmd);
 
@@ -449,7 +435,6 @@ do_open_directory (GnomeVFSMethod *method,
 	handle->type = SSH_LIST;
 	*method_handle = (GnomeVFSMethodHandle *)handle;
 
-	return result;
 	return GNOME_VFS_OK;
 }
 
@@ -461,8 +446,6 @@ do_close_directory (GnomeVFSMethod *method,
 	return ssh_destroy ((SshHandle *)method_handle);
 }
 
-#define LINE_LENGTH 4096 /* max line length we'll grok */
-
 static GnomeVFSResult 
 do_read_directory (GnomeVFSMethod *method,
 		   GnomeVFSMethodHandle *method_handle,
@@ -470,10 +453,10 @@ do_read_directory (GnomeVFSMethod *method,
 		   GnomeVFSContext *context)
 {
 	GnomeVFSResult result = GNOME_VFS_OK;
-	char line[LINE_LENGTH];
+	char line[LINE_LENGTH+1];
 	char c;
 	int i=0;
-	GnomeVFSFileSize j;
+	GnomeVFSFileSize bytes_read;
 	struct stat st;
 	char *tempfilename, *filename, *linkname;
 
@@ -482,11 +465,12 @@ do_read_directory (GnomeVFSMethod *method,
 		filename = NULL;
 		linkname = NULL;
 		i = 0;
-		j = 0;
+		bytes_read = 0;
 
 		while (i<LINE_LENGTH) {
-			result = ssh_read ((SshHandle *)method_handle, &c, 1, &j);
-			if (j == 0 || c == '\r' || c == '\n') {
+			result = ssh_read ((SshHandle *)method_handle, &c,
+					   sizeof(char), &bytes_read);
+			if (bytes_read == 0 || c == '\r' || c == '\n') {
 				break;
 			}
 
@@ -497,7 +481,9 @@ do_read_directory (GnomeVFSMethod *method,
 			line[i] = c;
 			i++;
 		}
-
+		/* Here we can have i == LINE_LENGTH which explains 
+		 * why the size of line is LINE_LENGTH+1
+		 */
 		line[i] = 0;
 		if (i == 0) {
 			return GNOME_VFS_ERROR_EOF;
@@ -556,12 +542,13 @@ do_get_file_info (GnomeVFSMethod *method,
 	char *cmd = NULL;
 	GnomeVFSResult result;
 	char *name;
+	char *quoted_name;
 
-	/* FIXME: escape for shell */
 	name = gnome_vfs_unescape_string (uri->text, G_DIR_SEPARATOR_S);
+	quoted_name = g_shell_quote (name);
 
-	if (strlen (name) > 0) {
-		cmd = g_strdup_printf ("ls -ld '%s' 2>&1", name);
+	if (*name != '\0') {
+		cmd = g_strdup_printf ("ls -ld %s 2>&1", quoted_name);
 	} else {
 		cmd = g_strdup_printf ("ls -ld '/' 2>&1");
 	}
@@ -569,6 +556,7 @@ do_get_file_info (GnomeVFSMethod *method,
 	result = ssh_connect (&handle, uri, cmd);
 	g_free (cmd);
 	g_free (name);
+	g_free (quoted_name);
 
 	if (result != GNOME_VFS_OK) {
 		return result;
@@ -595,15 +583,16 @@ do_make_directory (GnomeVFSMethod *method,
 	char *cmd = NULL;
 	GnomeVFSResult result;
 	char *name;
+	char *quoted_name;
 
-	/* FIXME: escape for shell */
 	name = gnome_vfs_unescape_string (uri->text, G_DIR_SEPARATOR_S);
+	quoted_name = g_shell_quote (name);
 
-	/* FIXME: escape for shell */
-	cmd = g_strdup_printf ("mkdir '%s'", name);
+	cmd = g_strdup_printf ("mkdir %s", quoted_name);
 	result = ssh_connect (&handle, uri, cmd);
 	g_free (cmd);
 	g_free (name);
+	g_free (quoted_name);
 
 	if (result != GNOME_VFS_OK) {
 		return result;
@@ -623,16 +612,18 @@ do_remove_directory (GnomeVFSMethod *method,
 	char *cmd = NULL;
 	GnomeVFSResult result;
 	gchar *name;
+	gchar *quoted_name;
 
 	name = gnome_vfs_unescape_string (uri->text, G_DIR_SEPARATOR_S);
 	if (name == NULL)
 		return GNOME_VFS_ERROR_INVALID_URI;
+	quoted_name = g_shell_quote (name);
 
-	/* FIXME: escape for shell */
-	cmd = g_strdup_printf ("rm -rf '%s'", name);
+	cmd = g_strdup_printf ("rm -rf %s", quoted_name);
 	result = ssh_connect (&handle, uri, cmd);
 	g_free (cmd);
 	g_free (name);
+	g_free (quoted_name);
 
 	if (result != GNOME_VFS_OK) {
 		return result;
@@ -652,16 +643,18 @@ do_unlink (GnomeVFSMethod *method,
 	char *cmd = NULL;
 	GnomeVFSResult result;
 	gchar *name;
+	gchar *quoted_name;
 
 	name = gnome_vfs_unescape_string (uri->text, G_DIR_SEPARATOR_S);
 	if (name == NULL)
 		return GNOME_VFS_ERROR_INVALID_URI;
+	quoted_name = g_shell_quote (name);
 
-	/* FIXME: escape for shell */
-	cmd = g_strdup_printf ("rm -rf '%s'", name);
+	cmd = g_strdup_printf ("rm -rf %s", quoted_name);
 	result = ssh_connect (&handle, uri, cmd);
 	g_free (cmd);
 	g_free (name);
+	g_free (quoted_name);
 
 	if (result != GNOME_VFS_OK) {
 		return result;
@@ -692,6 +685,8 @@ do_set_file_info (GnomeVFSMethod *method,
 		char *encoded_dir;
 		char *dir;
 		char *new_name;
+		char *quoted_full_name;
+		char *quoted_new_name;
 
 		encoded_dir = gnome_vfs_uri_extract_dirname (uri);
 		dir = gnome_vfs_unescape_string (encoded_dir, G_DIR_SEPARATOR_S);
@@ -710,32 +705,35 @@ do_set_file_info (GnomeVFSMethod *method,
 		}
 
 		/* FIXME: escape for shell */
-		cmd = g_strdup_printf ("mv '%s' '%s'", full_name,
-				       new_name);
+		quoted_new_name = g_shell_quote (new_name);
+		quoted_full_name = g_shell_quote (full_name);
+		cmd = g_strdup_printf ("mv %s %s", quoted_full_name,
+				       quoted_new_name);
 		result = ssh_connect (&handle, uri, cmd);
 		g_free (cmd);
 		g_free (dir);
 		g_free (new_name);
+		g_free (quoted_new_name);
+		g_free (quoted_full_name);
+		g_free (full_name);
+
+		if (result != GNOME_VFS_OK) {
+			return result;
+		}
 
 		/* Read all info from remote host */
 		while (1) {
 			char c;
 			GnomeVFSResult res;
-			GnomeVFSFileSize j;
-			res = ssh_read (handle, &c, 1, &j);
-			if (j == 0 || res != GNOME_VFS_OK)
+			GnomeVFSFileSize bytes_read;
+			res = ssh_read (handle, &c, 1, &bytes_read);
+			if (bytes_read == 0 || res != GNOME_VFS_OK)
 				break;
 		}
 
 		ssh_destroy (handle);
-
-		if (result != GNOME_VFS_OK) {
-			g_free (full_name);
-			return result;
-		}
 	}
 
-	g_free (full_name);
 	return result;
 }
 
